import unittest
from Products.Silva.tests.SilvaTestCase import SilvaTestCase

from DateTime import DateTime
from Products.ParsedXML.ParsedXML import ParsedXML

from Products.SilvaReferenceChecker.ReferenceChecker import ReferenceInfo
from Products.SilvaReferenceChecker.utils import get_references
from utils import assertEqualElements


class ReferenceTestCase(SilvaTestCase):
    """ test that the reference aggregation does actually collect
    the right references.
    the tests are only written for documents and "special content"
    objects, as for the "normal content" object (i,.e. with parsed xml
    versions) the content check should be good enough.
    """
        
    def afterSetUp(self):

        self.doc1=self.add_document(self.root, 'test_doc1','Test doc')
        self.folder = self.add_folder(self.root, 'test_folder', 'Test Folder')
        self.ghost = self.add_ghost(self.folder, 'test_ghost1',
                                    '/test_folder/test_doc2')

    def _publish(self, obj):
        assertTrue = self.assert_
        if obj.implements_container():
            obj = obj.get_default()
        assertTrue(obj.get_editable())
        obj.set_unapproved_version_publication_datetime(DateTime()-1)
        obj.approve_version()
        assertTrue(obj.get_viewable())
        

    def test_Document(self):
        """ test links in a document """

        # This document should contain the following references:
        # - a normal internal link in text,
        # - an external link,
        # - an image (with surrounding link, count as two references)
        # - a code element
        # Total: 5
        # XXX: maybe test for links in nested structures, e.g. table cells?

        xml = """\
        <doc>
        <p>This is a link <link url="/test_doc2">Ok</link> here</p>
        <p><link url="http://zope.org">External link</link></p>
        <image link="/image_link" path="/root/test_image" />
        <code path="/root/code_source/some_script" />
        </doc>
        """

        self.doc1.get_editable().content.manage_edit(xml)
        self._publish(self.doc1)

        expected = (
            ReferenceInfo(self.doc1, '/test_doc2'),
            ReferenceInfo(self.doc1, 'http://zope.org'),
            ReferenceInfo(self.doc1, '/image_link'),
            ReferenceInfo(self.doc1, '/root/test_image',
                          path_info=ReferenceInfo.PATH_ZOPE_ROOT,
                          ref_type=ReferenceInfo.REF_INTERN,
                          restriction=ReferenceInfo.RESTRICT_IMAGE),
            ReferenceInfo(self.doc1, '/root/code_source/some_script',
                          path_info=ReferenceInfo.PATH_ZOPE_ROOT,
                          ref_type=ReferenceInfo.REF_INTERN,
                          restriction=ReferenceInfo.RESTRICT_CODE),
            )
        references = get_references(self.doc1.get_viewable())
        
        #print references
        #print expected
        #self.assertEquals(str(expected), str(references))
        assertEqualElements(expected, references)

        # do the same for empty references;
        # they should be aggregated as well here
        xml = """\
        <doc>
        <p>This is a link <link url="">Ok</link> here</p>
        <p><link url="">Second link</link></p>
        <image></image>
        <code></code>
        </doc>
        """

        self.doc1.create_copy()
        self.doc1.get_editable().content.manage_edit(xml)

        expected = (
            ReferenceInfo(self.doc1, '',
                          ref_type=ReferenceInfo.REF_UNKNOWN),
            ReferenceInfo(self.doc1, '',
                          ref_type=ReferenceInfo.REF_UNKNOWN),
            ReferenceInfo(self.doc1, '',
                          path_info=ReferenceInfo.PATH_CONTAINER,
                          ref_type=ReferenceInfo.REF_INTERN,
                          restriction=ReferenceInfo.RESTRICT_IMAGE),
            ReferenceInfo(self.doc1, '',
                          path_info=ReferenceInfo.PATH_ZOPE_ROOT,
                          ref_type=ReferenceInfo.REF_INTERN,
                          restriction=ReferenceInfo.RESTRICT_CODE),
            )

        references = get_references(self.doc1.get_previewable())
        assertEqualElements(expected, references)
        
        self._publish(self.doc1)
        references = get_references(self.doc1.get_viewable())
        assertEqualElements(expected, references)

    def test_ghost(self):
        # no references for the ghost itself
        references = get_references(self.ghost)
        assertEqualElements((), references)
        
        self._publish(self.ghost)
        expected =( ReferenceInfo(self.ghost.get_viewable(),
                                  '/test_folder/test_doc2',
                                  restriction=ReferenceInfo.RESTRICT_CONTENT,
                                  ref_type=ReferenceInfo.REF_INTERN,
                                  path_info=ReferenceInfo.PATH_ZOPE_ROOT) ,)
        references = get_references(self.ghost.get_viewable())
        assertEqualElements(expected, references)
        


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ReferenceTestCase, 'test'))
    return suite        

def main():
    unittest.TextTestRunner(verbosity=1).run(test_suite())

if __name__ == '__main__':
    main()
